/*******************************************************************************
 * Copyright 2002 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
using System;
using System.Reflection;
using System.Collections;

class CEscalationExample
{
	// Private data members
	private const string escalationPriority = "Immediate";
	private long cookie;
	private ttsoapcgi soapServer;
	private DateTime  comparisonDate;//value used as part of determining if a defect should be escalated

	/******************************************************************************
	 * Main
	 */
	[STAThread]
	static void Main(string[] args)
	{
 		// Use the argument verifier class to get and verify the command line arguments
		CArguments arguments = new CArguments(ref args);
		if ( arguments.valid )
		{
			// Create a new instance of our class
			CEscalationExample example = new CEscalationExample();
			try 
			{
				// Verify that the database specified on the command line exists
				if (example.VerifyDatabase(arguments.database))
				{
					// Connect to the TestTrack Pro SOAP CGI
					example.Connect( arguments.database, arguments.username, arguments.password );
					// Verify that we can edit defects and that the priority to escalate to exists
					if ( example.CanEditDefects() )
					{
						if ( example.DoesEscalationPriorityExist() )
							example.EscalateDefects();
						else
							Console.WriteLine("The priority, {0}, is not present on the database. Unable to escalate defects.", arguments.database);
					}
					else
						Console.WriteLine("You do not have permission to edit defects.");
				}
				else
					Console.WriteLine("Error: The database \"{0}\" doesn't exist.", arguments.database );
			}
			catch( System.Exception e )
			{
				Console.WriteLine("Error: {0}", e.Message);
			}
			finally
			{
				// Ensure that we are disconnected
				example.Disconnect();
			}
		}
		else
			ShowUsage();
	}

	/******************************************************************************
	 *	CEscalationExample - Create new ttsoapcgi and comparison date objects
	 */
	CEscalationExample()
	{
		soapServer = new ttsoapcgi();
		comparisonDate = new DateTime( DateTime.Now.AddMonths(-1).Ticks );
	}

	/******************************************************************************
	 * ~CEscalationExample - Disconnect
	 */
	~CEscalationExample()
	{
		// Ensure that we're disconnected
		Disconnect();
	}

	/******************************************************************************
	 * ShowUsage - Show what arguments are used to invoke this program
	 */
	static void ShowUsage()
	{
		Module module = Assembly.GetExecutingAssembly().GetModules()[0];
		Console.WriteLine("\r\nUsage: {0} -u username [-p password] -d database", module.Name);
		Console.WriteLine("Example 1: {0} -u admin -p secret -d TestDB", module.Name );
		Console.WriteLine("Example 2: {0} -u admin -d \"Sample Database\"", module.Name);
	}

	/******************************************************************************
	 * Connect - Connects to a the given database using the supplied user name and
	 *           password
	 */
	public void Connect(string database, string username, string password)
	{
		cookie = soapServer.DatabaseLogon(database, username, password);
	}

	/******************************************************************************
	 * Disconnect - If we're currently connected to a database, disconnect
	 */
	public void Disconnect()
	{
		if ( cookie != 0 )
		{
			try 
			{
				long temp = cookie;
				cookie = 0;
				soapServer.DatabaseLogoff( temp );
			}
			catch (System.Exception e)
			{
				Console.WriteLine( "Error while attempting to disconnect, error message is: {0}", e.Message );
			}
		}
	}

	/******************************************************************************
	 * VerifyDatabase - Verifies that the database specified is on the server
	 */
	public bool VerifyDatabase(string dbName)
	{
		bool result = false;
		CDatabase[] databaseList = soapServer.getDatabaseList();
		foreach ( CDatabase db in databaseList )
		{
			if ( db.name == dbName )
			{
				result = true;
				break;
			}
		}
		return result;
	}

	/******************************************************************************
	 *	CanEditDefects - Verifies that the logged in user can edit defects
	 */
	public bool CanEditDefects()
	{
		bool bResult = false;
		CDatabaseTable[] tables = soapServer.getTableList(cookie);
		foreach ( CDatabaseTable table in tables )
		{
			if ( table.name == "Defect" )
			{
				bResult = true;
				break;
			}
		}
		return bResult;
	}

	/******************************************************************************
	 * DoesEscalationPriorityExit - Verifies that escalationPriority exists as
	 *                              a priority in the database
	 */
	public bool DoesEscalationPriorityExist()
	{
		bool bResult = false;
		CFieldValue[] fieldValues = soapServer.getDropdownFieldValuesForTable(cookie, "Defect", "Priority");
		foreach( CFieldValue fieldValue in fieldValues )
		{
			if ( fieldValue.value == escalationPriority )
			{
				bResult = true;
				break;
			}
		}
		return bResult;
	}

	/******************************************************************************
	 *	EscalateDefects - Escalates all defects to the "Immediate" priority in the
	 *                    current database that meet all of these criteria:
	 *                    1. State contains "Open"
	 *                    2. Defect is at least a month old, or older
	 *                    3. Defect isn't marked with the "Immediate" priority
	 */
	public void EscalateDefects()
	{
		CRecordListSoap list = GetDefectList();
		if ( list != null && list.records.Length > 0 )
		{
			foreach( CRecordRowSoap record in list.records )
			{
				// See if this defect matches our search criteria
				if ( (record.row[2].value.StartsWith("Open")) &&
					  (IsDateAtLeastOneMonthOld(ref record.row[3].value)) &&
					  (record.row[4].value != "Immediate" ) )
				{
					try 
					{
						// Note: It is possible that the defect hasn't yet been assigned
						//       a defect number, use 0 in this case
						long defectNumber = record.row[0].value == "-" ? 0 : Convert.ToInt64(record.row[0].value);
						CDefect defect = soapServer.editDefect( cookie, defectNumber, record.row[1].value, false );
						defect.priority = "Immediate";
						soapServer.saveDefect( cookie, defect );
					}
					catch ( System.Exception e )
					{
						Console.WriteLine( "Error occurred while attempting to edit defect number \"{0}\" with summary \"{1}\"",
											record.row[0].value,
											record.row[1].value);
						Console.WriteLine( "Error message: \"{0}\"\r\n", e.Message);
					}
				}
			}
		}
	}

	/******************************************************************************
	 * GetDefectList - Returns an array of defects on the current datbase logged
	 *                 into.The columns in the returned list will be defect number,
	 *                 summary, status, date entered, and priority
	 */
	public CRecordListSoap GetDefectList()
	{
		// Create a CTableColumn array
		CTableColumn[] columnsToReturn = new CTableColumn[5];
		for( uint i = 0 ; i < 5 ; i++ )
			columnsToReturn[i] = new CTableColumn();
		columnsToReturn[0].name = "Number";
		columnsToReturn[1].name = "Summary";
		columnsToReturn[2].name = "Status";
		columnsToReturn[3].name = "Date Entered";
		columnsToReturn[4].name = "Priority";
		
		return soapServer.getRecordListForTable( cookie, "Defect", "", columnsToReturn );
	}

	/******************************************************************************
	 * IsDateAtLeastOneMonthOld - Returns true if the given string representation
	 * of a date is a month or more old
	 */
	public bool IsDateAtLeastOneMonthOld( ref string date )
	{
		bool result = false;
		if ( date == null || date.Length == 0 )
			result = true;
		else
		{
			if ( Convert.ToDateTime( date ) <= comparisonDate )
				result = true;
		}
		return result;
	}

	#region CArguments Utility Class
	//##########################################################################
	//#	CArguments	- Private utility class to get and verify command line args.
	//##########################################################################
	private class CArguments
	{
		public CArguments(ref string[] args)
		{
			valid = false;
			if ( args.Length == 6 || args.Length == 4 )
			{
				if (args[0] == "-u" && args[2] == "-p" && args[4] == "-d")
				{
					username = args[1];
					password = args[3];
					database = args[5];
				}
				else if (args[0] == "-u" && args[2] == "-d")
				{
					username = args[1];
					database = args[3];
				}
				else
				{
					Console.WriteLine("Error: Invalid command line options.");
					return;
				}
				if ( username.Length != 0 && database.Length != 0 )
					valid = true;
				else if ( username.Length == 0 && database.Length == 0 )
					Console.WriteLine("Error: User name and database name are blank.");
				else if ( username.Length == 0 )
					Console.WriteLine("Error: User name is blank.");
				else//(database.Length == 0 )
					Console.WriteLine("Error: Database name is blank.");
			}
			else if ( args.Length != 0 )
				Console.WriteLine(@"Error: Invalid number and/or type of arguments");
		}
		public readonly string username;
		public readonly string password;
		public readonly string database;
		public readonly bool valid;		//if true then the values above are valid
	}
	#endregion
}